var Settings = (function() {
  var $settingsLink = null;
  var $settingsTabContainer = null;
  var themeObject = {
    dark: 'twilight',
    light: 'chrome'
  };

  var Service = null;
  var lsCurrentTheme = 'currentTheme';
  var possibleThemesArr = ['dark', 'light'];
  var $wrapper = null;
  var $html = null;
  var $bodyLoadingSymbol = null;
  var $somoDrpDwn = null;


  function init() {
    $settingsLink = $('#settingsLink');
    Service = Services(
      PBConstants.BASE_API_URL,
      PBConstants.MARKET_CAP_API_URL
    );
    $settingsTabContainer = $('#settingsTabContainer');
    $wrapper = $('#wrapper');
    $bodyLoadingSymbol = $('#bodyLoadingSymbol');
    $somoDrpDwn = $('#switchSom');
    $html = $('html');
    addEvents();
  }

  function hideBodyLoadingSymbol() {
    $bodyLoadingSymbol.hide();
    $wrapper.show();
  }

  function setTheme(isPageLoad, serverData) {
    var storedTheme = localStorage.getItem(Service.Normalizer.generateLocalStorageKey(lsCurrentTheme));
    // On page load we have to get theme from local storage current theme and show the portal in that theme.
    if (isPageLoad) {
      // If local storage theme is not set then wait untill first API call completes.
      if (storedTheme) {
        changeTheme(storedTheme);
        hideBodyLoadingSymbol();
      }
      return;
    }

    // This will get executed after every API call.
    if (serverData && serverData.settings) {
      // If server data has skin property and local storage theme value and API skin value is not same
      // then show portal in API skin color and update local storage theme value.
      var currentTheme = serverData.settings.skin;
      if (
        currentTheme &&
        currentTheme === storedTheme &&
        possibleThemesArr.indexOf(currentTheme) !== -1
      ) {
        return;
      }
      if (
        currentTheme &&
        currentTheme !== storedTheme &&
        possibleThemesArr.indexOf(currentTheme) !== -1
      ) {
        localStorage.setItem(Service.Normalizer.generateLocalStorageKey(lsCurrentTheme), serverData.settings.skin);
        hideBodyLoadingSymbol();
        changeTheme(currentTheme);
        return;
      }
    }

    // If API call fails then show portal in default theme.
    localStorage.setItem(Service.Normalizer.generateLocalStorageKey(lsCurrentTheme), PBConstants.DEFAULT_THEME);
    hideBodyLoadingSymbol();
  }

  // To change the theme of portal on function call.
  function changeTheme(themeName) {
    var themeCustomHref =
      THEME_CSS_FILE_PATH + themeName + '/custom-' + themeName + '.css';

    // To get the href of the present theme css link.
    var hrefOld = $('link#customTheme').attr('href');
    var hrefOldArr = hrefOld ? hrefOld.split('?') : [];
    if (hrefOldArr[0] === themeCustomHref) {
      // To not change the theme if the current theme is same as the stored theme and will return back to the calling function.
      return;
    }

    // The theme will be applied based on the current theme.
    var newCustomTheme =
      '<link rel="stylesheet" type="text/css" href="' +
      themeCustomHref +
      '" id="customTheme">';
    $('link#customTheme').replaceWith(newCustomTheme);

    // To change the theme of configuration console.
    var editorTheme = themeObject[themeName]
      ? themeObject[themeName]
      : PBConstants.DEFAULT_EDITOR_THEME;
    DomHelper.setEditorTheme(editorTheme);
    return;
  }

  function addEvents() {
    $settingsLink.click(function() {
      $('body').toggleClass('right-bar-enabled');
      // Main content should not be scrollable when sidebar is opened.
      $html.toggleClass('no-scrollbar');
    });

    // Hide settings on click on body.
    $('body, #appNotificationContainer a').click(function(e) {
      // Return if already settings tab is hidden.
      if (!$wrapper.hasClass('right-bar-enabled')) {
        return;
      }
      // Should not hide settings when the current target is settings link or settings tab or stop button alert.
      if (
        !$settingsTabContainer.has(e.target).length &&
        !$settingsLink.has(e.target).length &&
        $settingsLink[0] !== e.target &&
        $settingsTabContainer[0] !== e.target &&
        !$('.stop-button-alert').has(e.target).length
      ) {
        $wrapper.removeClass('right-bar-enabled');
        $html.removeClass('no-scrollbar');
      }
    });

    $somoDrpDwn.change(function () {
      Service.saveSOMO($(this).val()).done(function (
        data,
        textStatus,
        jqXHR
      ) {
        if (jqXHR.status === PBConstants.SUCCESS_CODE) {
          toastr.success(i18next.t('constantMsgs.somoSuccess'));
        } else {
          toastr.info(i18next.t('constantMsgs.ajaxError'));
        }
      });
    });
  }



  return {
    init: init,
    setTheme: setTheme
  };
})();
